<?php
namespace Ak\Wp\Model\Entity;

use Ak\Orm\Mapping\DiscriminatorEntry;
use Ak\Wp\Model\MetaInterface;
use Ak\Wp\Model\PostInterface;
use DateTimeImmutable;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;

/**
 * Class Post
 *
 * @package Ak\Wp\Model\Entity
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/wp-model
 *
 * @ORM\Entity()
 * @ORM\InheritanceType("SINGLE_TABLE")
 * @ORM\DiscriminatorColumn(name="post_type", type="string")
 * @DiscriminatorEntry("post")
 * @ORM\Table(name="wp_posts", options={"charset": "utf8mb4", "collate": "utf8mb4_general_ci"})
 */
class Post extends MetaAwareEntity implements PostInterface
{
    /**
     * @ORM\Id()
     * @ORM\GeneratedValue(strategy="AUTO")
     * @ORM\Column(name="ID", type="bigint", options={"unsigned": true})
     */
    protected ?int $id = null;

    /**
     * @var DateTimeImmutable
     *
     * @ORM\Column(name="post_date", type="datetime_immutable")
     */
    protected DateTimeImmutable $createdAt;

    /**
     * @var string
     *
     * @ORM\Column(name="post_status", type="string")
     */
    protected string $status = 'publish';

    /**
     * @var string
     *
     * @ORM\Column(name="post_title", type="string")
     */
    protected string $title = '';

    /**
     * @var string
     *
     * @ORM\Column(name="post_content", type="text")
     */
    protected string $content = '';

    /**
     * @var MetaInterface[]|Collection
     *
     * @ORM\OneToMany(targetEntity="Ak\Wp\Model\Entity\PostMeta", mappedBy="post", cascade={"persist", "remove"})
     */
    protected $metas;

    /**
     * Post constructor.
     */
    public function __construct()
    {
        parent::__construct();

        $this->createdAt = new DateTimeImmutable();
        $this->metas = new ArrayCollection();
    }

    /**
     * @return DateTimeImmutable
     */
    public function getCreatedAt(): DateTimeImmutable
    {
        return $this->createdAt;
    }

    /**
     * @return string
     */
    public function getStatus(): string
    {
        return $this->status;
    }

    /**
     * Vraća naslov ovog posta.
     *
     * @return string
     */
    public function getTitle(): string
    {
        return $this->title;
    }

    /**
     * Postavlja naslov ovog posta.
     *
     * @param string $title
     */
    public function setTitle(string $title): void
    {
        $this->title = $title;
    }

    /**
     * Vraća sadržaj posta.
     *
     * @return string
     */
    public function getContent(): string
    {
        return $this->content;
    }

    /**
     * Postavlja sadržaj ovog posta.
     *
     * @param string $content
     */
    public function setContent(string $content): void
    {
        $this->content = $content;
    }

    /**
     * @param string $key
     * @return MetaInterface
     */
    protected function createMeta(string $key): MetaInterface
    {
        return new PostMeta($this, $key);
    }
}
